// AUTORI: David Pilar, Marek Vojtek


// lex.cpp: implementation of the lex class.
//
//////////////////////////////////////////////////////////////////////

#include "lex.h"

// otevreni zdrojoveho souboru
boolean lex::otevriSoubor(char *nazev)
{
	if ((soubor = fopen (nazev, "rt")) == NULL)
	{
		cout << "Vstupni soubor se nepodarilo otevrit." << endl;
		return FALSE;
	}
	
	return TRUE;
}

// zavreni zdrojoveho souboru
boolean lex::zavriSoubor()
{
	if (fclose (soubor) == EOF)
	{
		cout << "Vstupni soubor se nepodarilo zavrit." << endl;
		return FALSE;
	}

	return TRUE;
}

// nastaveni hodnoty promenne radek
void lex::nastavRadek(int hodnota)
{
	radek = hodnota;
}

// vraci nacteny symbol
char *lex::vratNactenySymbol()
{
	return hid;
}

// vraci booleanovou hodnotu
boolean lex::vratHodnotuBool()
{
	return hbool;
}

// vraci intovou hodnotu
int lex::vratHodnotuInt()
{
	return hint;
}

// vraci realnou hodnotu
double lex::vratHodnotuReal()
{
	return hreal;
}

// vraci hodnotu relacniho operatoru
REL_OP lex::vratHodnotuRelOp()
{
	return hrelop;
}

// vraci string hodnotu
char *lex::vratHodnotuString()
{
	return hstring;
}

// vraci cislo aktualniho cteneho radeku
int lex::vratRadek()
{
	return radek;
}

// cte vstupni soubor
int lex::ctiSoubor()
{
	int znak;

	strcpy (hid, "");

	do
	{
		if ((znak = fgetc (soubor)) != ' ')
		{
			if (znak == '\'')
			{
				while (((znak = fgetc (soubor)) != '\n')&&(znak != EOF));
				radek++;
			}
			else
			if (znak == EOF)
			{
				strcpy (hid, "EOF");
				return ENDOFF;
			}
			else
			if (znak == '+')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return PLUS;
			}
			else
			if (znak == '-')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return MINUS;
			}
			else
			if (znak == '*')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return KRAT;
			}
			else
			if (znak == '/')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return DELENO;
			}
			else
			if (znak == ',')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return CARKA;
			}
			else
			if (znak == ';')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return STREDNIK;
			}
			else
			if (znak == '=')
			{
				hid[0] = znak;
				hid[1] = '\0';
				hrelop = ROVNO;
				return ROVNASE;
			}
			else
			if (znak == '(')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return LZAVORKA;
			}
			else
			if (znak == ')')
			{
				hid[0] = znak;
				hid[1] = '\0';
				return PZAVORKA;
			}
			else
			if (znak == '\"')
			{
				nactiString();
				return STRING;
			}
			else
			if (isalpha(znak))
			{
				hid[0] = znak;
				hid[1] = '\0';
				return nactiSlovo();
			}
			else
			if (isdigit(znak))
			{
				hid[0] = znak;
				hid[1] = '\0';
				return nactiCislo();
			}
			else
			if (znak == '<')
			{
				hid[0] = znak;
				hid[1] = '\0';
				nactiMensi();
				return RELOP;
			}
			else
			if (znak == '>')
			{
				hid[0] = znak;
				hid[1] = '\0';
				nactiVetsi();
				return RELOP;
			}
			else
			{
				if (znak == '\n')
					radek++;
				if ((znak != ' ')&&(znak != '	')&&(znak != '\n'))
				{
					hid[0] = znak;
					hid[1] = '\0';
					return UNKNOWN;
				}
			}
		}
		

	} while (true);

}

// cte ze vstupniho souboru relacni operator zacinajici na <
void lex::nactiMensi()
{
	int znak;
	int pom;

	znak = getc (soubor);

	if (znak == '=')
	{
		pom = strlen(hid);
		hid[pom] = znak;
		hid[pom+1] = '\0';
		hrelop = MENSIROVNO;
	}
	else
		if (znak == '>')
		{
			pom = strlen(hid);
			hid[pom] = znak;
			hid[pom+1] = '\0';
			hrelop = NEROVNO;
		}
		else
		{
			hrelop = MENSI;
			ungetc (znak, soubor);
		}
}

// cte ze vstupniho souboru relacni operator zacinajici na >
void lex::nactiVetsi()
{
	int znak;
	int pom;

	znak = getc (soubor);

	if (znak == '=')
	{
		pom = strlen(hid);
		hid[pom] = znak;
		hid[pom+1] = '\0';
		hrelop = VETSIROVNO;
	}
	else
	{
		hrelop = VETSI;
		ungetc (znak, soubor);
	}
}

// cte ze vstupniho souboru retezec v uvozovkach
void lex::nactiString()
{
	int znak;
	int pom;

	strcpy (hstring, ""); // new

	while ((znak = getc (soubor)) != '\"')
	{
		if (znak == EOF)
		{
			chyba = 1;
			break;
		}
		if (znak == '\n')
			radek++;
		if (znak == '\\')
		{
			znak = getc(soubor);
			if (znak == 'n')
				znak = '\n';
			else
				if (znak == '\"')
					znak = '\"';

				else
				{
					ungetc(znak, soubor);
					znak = '\\';
				}
		}
		pom = strlen(hstring);
		hstring[pom] = znak;
		hstring[pom+1] = '\0';
	}
}

// nastavi cislo chyby (0... bez chyby)
void lex::nastavChybu(int hodnota)
{
	chyba = hodnota;
}

int lex::vratChybu()
{
	return chyba;
}

// nacte slovo ze souboru (slovo obsahuje pouze pismena a cislice, zacina pismenem)
int lex::nactiSlovo()
{
	int znak;
	int pom;
	unsigned int i;
	char slovo[DELKA];

	while (isalnum (znak = getc (soubor))||(znak == '_'))
	{
		pom = strlen(hid);
		hid[pom] = znak;
		hid[pom+1] = '\0';
	}

	ungetc (znak, soubor);

	strcpy (slovo, hid);

	for (i = 0; i < strlen(slovo); i++)
		slovo[i] = toupper (slovo[i]);

	if (strcmp (slovo, "DIM") == 0)
		return DIM;
	if (strcmp (slovo, "AS") == 0)
		return AS;
	if (strcmp (slovo, "LET") == 0)
		return LET;
	if (strcmp (slovo, "READ") == 0)
		return READ;
	if (strcmp (slovo, "WRITE") == 0)
		return WRITE;
	if (strcmp (slovo, "DIV") == 0)
		return DIV;
	if (strcmp (slovo, "MOD") == 0)
		return MOD;
	if (strcmp (slovo, "AND") == 0)
		return AND;
	if (strcmp (slovo, "OR") == 0)
		return OR;
	if (strcmp (slovo, "NOT") == 0)
		return NOT;
	if (strcmp (slovo, "INTEGER") == 0)
		return IDINT;
	if (strcmp (slovo, "REAL") == 0)
		return IDREAL;
	if (strcmp (slovo, "BOOLEAN") == 0)
		return IDBOOL;
	if (strcmp (slovo, "STRING") == 0)
		return IDSTR;
	if (strcmp (slovo, "TRUE") == 0)
	{
		hbool = TRUE;
		return BOOL;
	}
	if (strcmp (slovo, "FALSE") == 0)
	{
		hbool = FALSE;
		return BOOL;
	}
	if (strcmp (slovo, "IF") == 0)
		return IF;
	if (strcmp (slovo, "THEN") == 0)
		return THEN;
	if (strcmp (slovo, "ELSE") == 0)
		return ELSE;
	if (strcmp (slovo, "TRUNC") == 0)
		return TRUNC;
	if (strcmp (slovo, "ROUND") == 0)
		return ROUND;
	if (strcmp (slovo, "STR") == 0)
		return STR;

	return PROM;
}

int lex::nactiCislo()
{
	int znak;
	int pom;
	boolean jereal;
	
	jereal = FALSE;

	while ((isdigit(znak = getc (soubor))||(znak == '.')))
	{
		if (znak == '.')
		{
			if (jereal)
				break;
			else
				jereal = TRUE;
		}

		pom = strlen(hid);
		hid[pom] = znak;
		hid[pom+1] = '\0';
	}

	ungetc (znak, soubor);

	if (jereal)
	{
		hreal = atof(hid);
		return REAL;
	}
	else
	{
		hint = atoi (hid);
		return INT;
	}

}
